/******************************************************************************
*
* Wolfenstein 3D - For TI-89, TI-92+, and Voyage 200
* Test by Malcolm Smith and Scott Noveck, Game by id Software
*
* ("restrict player to come not too close" code in PlayerMove() developed by
*  Scott Noveck)
*
* demonstrates: (3) "come not too close to a wall" effect including autosliding
*                   if characters stucks
*
******************************************************************************/

#define USE_TI89
#define USE_TI92PLUS
#define USE_V200

#include "fat.h"
#include "extgraph.h"
#include "e1l1.h"
#include "e1l2.h"
#include "e1l3.h"
#include "e1l4.h"
//#include "testlev.h"
#include "i_wolfgfx1.h"
#include "i_wolfobj.h"
#include "defs.h"
#include "extrand.h"
#include "indices.h"
#include "spriteindices.h" //This is not the best approach.
//I should probably just make sure the sprites are added in order to the file.
#include "ceilfloor.h"
#include "floors.h"
#include "doors.h"
#include "player.h"
#include "sprite_init.h"
#include "sprite_exec.h"
#include "titlescreen.h"


//-----------------------------------------------------------------------------
// include the utility routines ...
//-----------------------------------------------------------------------------
#define NO_PLAYER_MOVE     // we don't use the default movement code
#define NO_COUNTER_SUPPORT // we use no counters in this demo
#include "utils.c"



//=============================================================================
// as usual ... our main function
//=============================================================================
void _main(void) {
    PADSTATES       key = 0;
    TEXCONFIG       _textures[TOTAL_TEXCONFIGS_IN_wolfgfx1];
    TEXCONFIG				_objects[TOTAL_TEXCONFIGS_IN_wolfobj];
    char *map;
    Player _player;
    LCD_BUFFER      lcd;
    HANDLE          texfile = 0, objfile = 0;
    unsigned char*  src     = NULL;
    short           oldfont;
    short           result;

    objects = _objects;
    textures = _textures;
    objectmap = e1l1_objects;
    ceilmode = DARK;
    floormode = LIGHT;
    player = &_player;

    FATCONFIG fc = {
        64,
        64,
        NULL,
        textures,
        0,
        sprites,
        0,   // start pos_x
        0,   // start pos_y
        0,    // orientation
        NULL, // destination plane0
        NULL, // destination plane1
        NULL, // background image
        0     // frame counter
    };

    //-------------------------------------------------------------------------
    // before we try to do anything else we have to initialize the engine
    // its only valid to proceed if FAT_LOADOKAY gets returned
    //-------------------------------------------------------------------------
    if ((result = FAT_LoadEngine()) != FAT_LOADOKAY) {
        switch(result) {
            case FAT_NOTGHOSTSPACE: ST_helpMsg("ERROR: client no exepacked!");break;
            case FAT_NOTFOUND:      ST_helpMsg("ERROR: no matching engine found");break;
            case FAT_LOCKFAILED:    ST_helpMsg("ERROR: locking engine failed");break;
            case FAT_OUTOFMEM:      ST_helpMsg("ERROR: out of memory");break;
            default:                ST_helpMsg("ERROR: unknown errorcode");break;
        }
        return;
    }

    //-------------------------------------------------------------------------
    // try to load the used textures ...
    //-------------------------------------------------------------------------
    if ((texfile = FAT_GetHandleOfFile("wolfgfx1"))) {
        if ((src = FAT_LockHandleOfFile(texfile))) {
            // load wall and door textures
            if (!FAT_LoadTextures(textures,src+2,0,TOTAL_TEXCONFIGS_IN_wolfgfx1)) {
                src = 0; // just to indicate something went wrong
            }
         }
    }

    //-------------------------------------------------------------------------
    // try to load the used objects ...
    //-------------------------------------------------------------------------
    if ((objfile = FAT_GetHandleOfFile("wolfobj"))) {
        if ((src = FAT_LockHandleOfFile(objfile))) {
            // load wall and door textures
            if (!FAT_LoadTextures(objects,src+2,0,TOTAL_TEXCONFIGS_IN_wolfobj)) {
                src = 0; // just to indicate something went wrong
            }
         }
    }

    //-------------------------------------------------------------------------
    // check if loading failed ...
    //-------------------------------------------------------------------------
    if (!texfile || !objfile|| !src) {
        FAT_UnloadEngine();
        if (texfile) FAT_UnlockHandleOfFile(texfile);
        if(objfile) FAT_UnlockHandleOfFile(objfile);
        ST_helpMsg("ERROR: WolfGfx1 or WolfObj");
        return;
    }

    LCD_save(lcd);

    InstallDummyHandlerForINTs();

    //-------------------------------------------------------------------------
    // initialize grayscales, clear both planes and set the destination plane
    // addresses for the FAT-Engine
    //-------------------------------------------------------------------------
    if (!GrayOn()) {
        RestoreHandlerForINTs();
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        FAT_UnlockHandleOfFile(objfile);
        ST_helpMsg("ERROR: out of mem");
        return;
    }

		if(!(sprites=malloc(MAXSPRITES*(sizeof(FATSPRITE)+sizeof(statobj)) + 1152*2 + 4096 +4096 +
			MAX_NR_DOORS*sizeof(DoorStruct))))
		{
				GrayOff();
        RestoreHandlerForINTs();
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        FAT_UnlockHandleOfFile(objfile);
        ST_helpMsg("ERROR: out of mem");
        return;
		}
		fc.sprites = sprites;
		sprites_extra = (statobj*)(sprites+MAXSPRITES);
		bg_img = (char*)(sprites_extra+MAXSPRITES);
		map = bg_img+1152*2;
		actorat = (map+4096);
		doordata = (DoorStruct*)(actorat+4096);
		fc.map_data = map;


    memcpy(map,e1l1_walls_and_floors,4096);

		IntroScreens();

    memset(GetPlane(0),255,LCD_SIZE);
    memset(GetPlane(1),255,LCD_SIZE);

    if (TI89) {
        display_offset  = 0;
        display_offsetx = 0;
        display_offsety = 0;
    }
    else {
        // TI92p or V200
        display_offset   = 14*30+4;
        display_offsetx  = 48;
        display_offsety  = 14;
    }

    fc.dest_plane0 = (unsigned short*)((char*)GetPlane(0)+display_offset);
    fc.dest_plane1 = (unsigned short*)((char*)GetPlane(1)+display_offset);

    oldfont=FontSetSys(F_4x6);

    ReloadLevel(&fc);
    SetUpCeilingAndFloor(&fc);

    FontSetSys(F_4x6);
    DrawGrayStr(100+display_offsetx,10+display_offsety,"Wolfenstein 68k",A_REVERSE);
		DrawGrayStr(100+display_offsetx,21+display_offsety,"Malcolm Smith &",A_REVERSE);
		DrawGrayStr(100+display_offsetx,28+display_offsety,"Scott Noveck",A_REVERSE);
		DrawGrayStr(100+display_offsetx,40+display_offsety,"v0.01 Internal",A_REVERSE);

    //-------------------------------------------------------------------------
    // the main rendering loop ...
    //-------------------------------------------------------------------------
    do {
            FAT_Render(&fc);
            CheckAreas(&fc);
            MoveDoors(&fc);
            MovePWalls(&fc);
            HideCloseAndFarSprites(&fc);
            CheckPlayerItemSpot(&fc);

            //printf_xy(0,0,"%i ",fc.nr_sprites);

					if ((key = FAT_GetPadState()))
					{
						if (key & PADSTATE_ESC)       break;
						if (key & PADSTATE_UPDOWN)
						{
							if(key & PADSTATE_KA)
								PlayerMove(&fc,key & PADSTATE_UP,0);
							PlayerMove(&fc,key & PADSTATE_UP,0);
						}
           if (key & PADSTATE_LEFTRIGHT)
           {
           	if(key & PADSTATE_KB)
           	{
           		if(key & PADSTATE_KA)
           			PlayerMove(&fc,key & PADSTATE_RIGHT,1);
           		PlayerMove(&fc,key & PADSTATE_RIGHT,1);
           	}
           	else PlayerTurn(&fc,key & PADSTATE_RIGHT,ANGLE_015/2);
           }
        }
				if(_keytest(RR_HOME))
				{
					while(_keytest(RR_HOME));
					OpenDoorForPlayer(&fc);
				}
				if(_keytest(RR_1))
				{
					memcpy(fc.map_data , e1l1_walls_and_floors,4096);
					objectmap = e1l1_objects;
					ReloadLevel(&fc);
				}
				if(_keytest(RR_2))
				{
					memcpy(fc.map_data , e1l2_walls_and_floors,4096);
					objectmap = e1l2_objects;
					ReloadLevel(&fc);
				}
				if(_keytest(RR_3))
				{
					memcpy(fc.map_data , e1l3_walls_and_floors,4096);
					objectmap = e1l3_objects;
					ReloadLevel(&fc);
				}
				if(_keytest(RR_4))
				{
					memcpy(fc.map_data , e1l4_walls_and_floors,4096);
					objectmap = e1l4_objects;
					ReloadLevel(&fc);
				}
				/*if(_keytest(RR_0))
				{
					memcpy(fc.map_data , testlev_walls_and_floors,4096);
					objectmap = testlev_objects;
					ReloadLevel(&fc);
				}*/

				/*if(_keytest(RR_POWER)) //Toggle the ceiling lights
				{
					if(fc.nr_sprites==spritecount) fc.nr_sprites-=lightcount;
					else fc.nr_sprites+=lightcount;
				}*/
    }
    while(1);

    //-------------------------------------------------------------------------
    // some clean stuff ...
    //-------------------------------------------------------------------------
    FAT_AllKeysReleased();
    GrayOff();
    LCD_restore(lcd);
    ST_helpMsg(FAT_GetPoweredByString());
    FAT_UnlockHandleOfFile(texfile);
    FAT_UnlockHandleOfFile(objfile);
    FAT_UnloadEngine();
    free(sprites);
    RestoreHandlerForINTs();
    OSqclear(kbd_queue());
    FontSetSys(oldfont);
}

